<?php
define('COOKIE_TIMEOUT', time() + (365 * 86400)); //現在日時+一年 (86400: 24時間*60分*60秒)
define('COOKIE_DOMAIN', '');
define('COOKIE_PATH', '/d/');
define('COOKIE_MSITE_KEY', 'msite'); //マーキングサイト。１文字目はデリミタ
define('COOKIE_DSITE_KEY', 'dsite'); //除外サイト。１文字目はデリミタ
define('COOKIE_SITE_MAX', 50);
/***********************************************************
 * Cookieの指定されたキーに格納されているサイト番号の一覧を返す。
 * ---------------------------------------------------------
 * [引数] $key: 格納されているCookieのキー。1文字目はデリミタになる
 * [戻り] 各サイト番号を１要素として格納した配列。該当する
 *        キーがCookieにない場合は空の配列。
 ***********************************************************/
function getSiteNumberFromCookie($key) {
 $site_number_list = array();
 if (isset($_COOKIE[$key])) {
  $delimiter = substr($key, 0, 1);
  $wk = explode($delimiter, $_COOKIE[$key]);
  foreach($wk as $num){
   if ((strlen($num) > 0) && ctype_digit($num)) $site_number_list[] = $num;
   if (count($site_number_list)>=COOKIE_SITE_MAX) break;
  }
 }
 return $site_number_list; //格納する際にソートするので、ここではソートしない
}
/***********************************************************
 * 配列にあるサイト番号の一覧を、昇順ソートしてCookieに格納する。
 * ---------------------------------------------------------
 * [引数] $key: 保存するCookieのキー。1文字目は連結文字列になる
 *        $array: サイト番号が格納されている配列
 * [戻り] なし
 ***********************************************************/
function saveSiteNumberToCookie($key, $array) {
 $connector = substr($key, 0, 1);
 sort($array, SORT_NUMERIC);
 setcookie($key,
  implode($connector, $array),
  COOKIE_TIMEOUT,
  COOKIE_PATH,
  COOKIE_DOMAIN
 );
}
/***********************************************************
 * 動作指定に即してマーク/マーク解除/除外/除外解除の処理を行ない、
 * Cookieを更新する。
 * ---------------------------------------------------------
 * @param string $action 動作指定。mm、mo、dm、do、のいずれか
 * @param string $site_id サイトID
 * @param array  &$markList マークサイトリスト
 * @param array  &$deselectList 除外サイトリスト
 * @return null（正常終了時）またはエラーメッセージ
 ***********************************************************/
function updateSiteNumber($action, $site_id, &$markList, &$deselectList) {
	if (!isDigit($site_id)) return '指定されたサイトIDは不正です。';
	if (strlen($action) != 2) return '有効なアクションではありません。';
	if ($action == 'mm') { //マーク
		if (!in_array($site_id, $markList)) {
			if (count($markList) >= COOKIE_SITE_MAX) return '既に'.COOKIE_SITE_MAX.'件マークされています。これ以上追加できません。';
			$markList[] = $site_id;
			saveSiteNumberToCookie(COOKIE_MSITE_KEY, $markList);
		}
	} else if ($action == 'mo') { //マーク解除
		if (in_array($site_id, $markList)) {
			$markList = array_diff($markList, array($site_id));
			saveSiteNumberToCookie(COOKIE_MSITE_KEY, $markList);
		}
	} else if ($action == 'dm') { //除外
		if (!in_array($site_id, $deselectList)) {
			if (count($deselectList) >= COOKIE_SITE_MAX) return '既に'.COOKIE_SITE_MAX.'件除外されています。これ以上追加できません。';
			$deselectList[] = $site_id;
			saveSiteNumberToCookie(COOKIE_DSITE_KEY, $deselectList);
		}
	} else if ($action == 'do') { //除外解除
		if (in_array($site_id, $deselectList)) {
			$deselectList = array_diff($deselectList, array($site_id));
			saveSiteNumberToCookie(COOKIE_DSITE_KEY, $deselectList);
		}
	} else {
		return '有効なアクションではありません。';
	}
	return null;
}
