<?php
require_once(dirname(__FILE__) . '/common.php');
define('HTTP_HEADER_403', 'HTTP/1.0 403 Forbidden');
/***********************************************************
 * SQLiteのSQLとしてエスケープするべき文字をエスケープする。
 * ---------------------------------------------------------
 * @param  string $str エスケープする文字列
 * @param  string $encoding 文字コード(既定値はUTF-8)
 * @return エスケープした文字列
 ***********************************************************/
function sqlescape($str, $escapechar='$', $encoding='UTF-8') {
	mb_regex_encoding($encoding);
	$escape = preg_match('/[%_]/', $str);
	if ($escape) $str = mb_ereg_replace('([%_'.$escapechar.'])', $escapechar.'\1', $str);
	return array('str'=>sqlite_escape_string($str), 'escape'=>$escape);
}
/***********************************************************
 * 文字列の末尾に改行コードをつけて出力する。
 * HTMLエスケープ処理は行なわない。
 * ---------------------------------------------------------
 * @param  string $str 出力する文字列
 * @return なし
 ***********************************************************/
function put($str) {
	echo $str, "\n";
}
/***********************************************************
 * trim関数の日本語対応版。文字列の先頭および末尾にある
 * ホワイトスペースを取り除く。ホワイトスペースと見なすのは
 * 半角空白([:space:])、ヌル文字、全角空白。[:space:]は
 * [ \t\n\r\x0b\x0c]と等価（\x0bは垂直タブ、\x0cはラインフィード）。
 * ---------------------------------------------------------
 * @param  string $str 文字列
 * @param  string $charlist 削除する文字の羅列(既定値はホワイトスペースとヌル文字)
 * @param  string $encoding 文字コード(既定値はUTF-8)
 * @return ホワイトスペースを取り除いた文字列
 ***********************************************************/
function mb_trim($str, $charlist='[:space:]\0　', $encoding='UTF-8'){
	mb_regex_encoding($encoding);
	return mb_ereg_replace("(^[$charlist]+|[$charlist]+$)", '', $str);
}
/***********************************************************
 * mb_trimのArray対応版。Stringに加え、Arrayのキーおよび値も
 * すべてトリミングする。String、Array以外の値はそのまま返す。
 * ---------------------------------------------------------
 * @param  string $obj 文字列または配列
 * @param  string $charlist 削除する文字の羅列(既定値はホワイトスペースとヌル文字)
 * @param  string $encoding 文字コード(既定値はUTF-8)
 * @return ホワイトスペースを取り除いた文字列または配列
 ***********************************************************/
function mb_trim_ex($obj, $charlist='[:space:]\0　', $encoding='UTF-8'){
	if (!$obj) return $obj;
	if (is_string($obj)) return mb_trim($obj);
	if (!is_array($obj)) return $obj;
	foreach($obj as $key=>$val) {
		if (is_string($val)) {
			$obj[$key] = mb_trim($val);
		} else if (is_array($val)) {
			$obj[$key] = mb_trim_ex($val);
		}
		if (is_string($key)) {
			$newkey = mb_trim($key);
			if ($newkey != $key) {
				$obj[$newkey] = $obj[$key];
				unset($obj[$key]);
			}
		}
	}
	return $obj;
}
/***********************************************************
 * 文字列から改行コードを除去する。
 * ---------------------------------------------------------
 * @param  string $str 文字列
 * @return 改行を除去した文字列
 ***********************************************************/
function removeLineFeedCode($str) {
	return preg_replace('/[\r\n]/', '', $str);
}
/***********************************************************
 * 文字列から制御コードを除去する。
 * 制御コードと見なす値は、0x00～0x1f、0x7f。
 * ---------------------------------------------------------
 * @param  string $str 文字列
 * @return 制御コードを除去した文字列
 ***********************************************************/
function removeControlCode($str) {
	return preg_replace('/[\x00-\x1f\x7f]/', '', $str);
}
/***********************************************************
 * 文字列が半角の数字だけで構成されているかチェックする。
 * null、空のStringはチェックNGとなる。
 * ---------------------------------------------------------
 * @param  string $str 文字列
 * @return true:チェックOK、false:チェックNG
 ***********************************************************/
function isDigit($str) {
	return preg_match('/^\d+$/D', $str);
}
/***********************************************************
 * リファラをチェックする。不正なリファラと判断した場合は
 * HTTPステータス403を発行する。
 * ---------------------------------------------------------
 * @param  boolean $blankOk 空のリファラを許容するか(規定値はfalse)
 * @param  string  $path パス。指定する場合は末尾に要スラッシュ(既定値は'd/')
 * @return true:正常と見なしたリファラ、false:不正と見なしたリファラ
 ***********************************************************/
function checkReferer($blankOk=false, $path='d/') {
	$ref = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : null;
	if (!strlen($ref)) {
		if ($blankOk) return true; //空のリファラを許容
		header(HTTP_HEADER_403);
		return false;
	}
	$valid_ref = (DEBUG ? VALID_DOMAIN_DEBUG : VALID_DOMAIN).$path;
	if (substr($ref, 0, strlen($valid_ref)) != $valid_ref) { //TODO ドメインは小文字に変換して比較
		header(HTTP_HEADER_403);
		return false;
	}
	return true;
}
/***********************************************************
 * メッセージを表示してスクリプトを終了する。
 * ---------------------------------------------------------
 * @param  string or integer $status メッセージまたは終了ステータス(規定値は0)
 * @return なし
 ***********************************************************/
function abort($status=0) {
	if (is_string($status) && !headers_sent()) {
		header('Content-Type: text/plain; charset=UTF-8');
	}
	die($status);
}
