<?php
require_once(dirname(__FILE__) . "/../lib/util.php");
//このファイルはUTF-8で記載すること
class Condition {
	//GET時の検索オプション条件
	private static $search_get_items = array('mda','styp','ctg','mod','scl','ctn','rst','cntr','chr','cptp','cp','sm','uk');
	//POST時の検索オプション条件
	private static $search_post_items = array('mda','styp','ctg','mod','scl','ctn','rst','cntr','chr','cptp','cp','sm','uk');
	//検索基本項目規定値（チェックボックスはOFF時は渡されないため、OFF値を規定値とする）
	private $base_default = array(
		 'shwd'=>'' //検索語
		,'ngwd'=>'' //除外語
		,'and'=>'1' //1:AND、0:OR
		,'r'=>'a'   //range:検索範囲。a:all、u:upd、d:day
		,'r_dv'=>'7' //range=dの日数
		,'sw_b'=>'0' //banner:バナー
		,'sw_o'=>'0' //outline:概要
		,'sw_d'=>'0' //desc:紹介文
		,'sw_h'=>'0' //hist:更新履歴
		,'o'=>'u'  //order:表示順。u:upd、e:entry、n:name、r:rand
		,'rv'=>'0' //reverse:逆順か。0:昇順、1:降順
		,'c'=>'10' //count:1ページの表示最大件数
		,'n_c'=>'0' //新count:1ページの表示最大件数(検索結果画面での件数変更時のみ)
		,'p'=>'0'  //page:表示するページ番号-1
		,'a'=>'sh' //act:アクション種別
		,'f'=>'' //from:前画面。詳細情報画面でのみ使用
	);
	//検索基本項目
	public $base = null;
	//検索オプション項目（ok:検索条件、ng:除外条件）
	public $option = array('ok'=>array(), 'ng'=>array());
	/***********************************************************
	 * 規定値にリクエストパラメータを上書きする形でマージする。
	 * その際、ホワイトスペースのトリミングおよび不正な値の
	 * 規定値による上書きも行なう。
	 * ※ユーザの利便性を鑑み、不正な値が含まれていてもエラー
	 * 　メッセージを出して弾くのではなく、極力検索を行なう。
	 * ---------------------------------------------------------
	 * @param  なし
	 * @return なし
	 ***********************************************************/
	public function margeCondition() {
		$this->base = $this->base_default;
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			$this->margePostCondition();
		} else if ($_SERVER['REQUEST_METHOD'] === 'GET') {
			$this->margeGetCondition();
		} else {
			return;
		}
		$this->checkBaseCondition();
		$this->checkOptCondition();
	}
	/***********************************************************
	 * POSTされた値をトリミングして、検索基本項目および
	 * 検索オプション項目に格納する。
	 * ---------------------------------------------------------
	 * @param  なし
	 * @return なし
	 ***********************************************************/
	private function margePostCondition() {
		foreach($_POST as $key=>$val) {
			$val = mb_trim_ex($val);
			if (array_key_exists($key, $this->base)) { //検索基本項目にある
				if (is_string($val)) $this->base[$key] = $val; //検索基本項目に追加
			} else if (in_array($key, self::$search_post_items)) { //検索オプション条件一覧にある
				if (is_array($val)) $this->option['ok'][$key] = $val; //検索条件に追加
			} else if (preg_match('/^(sm|uk)_(o|n)$/', $key, $matches)) { //受け攻め指定
				if ($_POST[$matches[1].'_id']) { //キャラ指定あり
					if ($matches[2] == 'o') {
						$this->option['ok'][$matches[1]] = array($_POST[$matches[1].'_id']);
					} else {
						$this->option['ng'][$matches[1]] = array($_POST[$matches[1].'_id']);
					}
				}
			} else {
				if (preg_match('/^(.+)_n$/', $key, $matches)) { //_nを除くと検索オプション条件一覧にある
					if (in_array($matches[1], self::$search_post_items)) {
						if (is_array($val)) $this->option['ng'][$matches[1]] = $val; //除外条件に追加
					}
				}
			}
		}
	}
	/***********************************************************
	 * GETされた値をトリミングして、検索基本項目および
	 * 検索オプション項目に格納する。
	 * ---------------------------------------------------------
	 * @param  なし
	 * @return なし
	 ***********************************************************/
	private function margeGetCondition() {
		foreach($_GET as $key=>$val) {
			if (!is_string($val)) continue;
			$val = mb_trim($val);
			if (array_key_exists($key, $this->base)) { //検索基本項目にある
				$this->base[$key] = $val; //検索基本項目に追加
			} else if (in_array($key, self::$search_get_items)) { //検索オプション条件一覧にある
				$this->option['ok'][$key] = explode('x',$val); //xで分割して検索条件に格納
			} else {
				if (preg_match('/^(.+)_n$/', $key, $matches)) { //_nを除くと検索オプション条件一覧にある
					if (in_array($matches[1], self::$search_get_items)) {
						$this->option['ng'][$matches[1]] = explode('x',$val); //xで分割して除外条件に格納
					}
				}
			}
		}
		if (isset($_GET['a']) && ($_GET['a']=='lk')) { //検索メニュー項目リンク
			$this->base['sw_b'] = '1';
			$this->base['sw_o'] = '1';
			$this->base['sw_d'] = '1';
			$this->base['sw_h'] = '1';
		}
	}
	/***********************************************************
	 * 検索基本項目に格納された値をチェックし、不正な内容なら
	 * 規定値で上書きする。
	 * ---------------------------------------------------------
	 * @param  なし
	 * @return なし
	 ***********************************************************/
	private function checkBaseCondition() {
		$this->base['shwd'] = removeControlCode($this->base['shwd']);
		$this->base['ngwd'] = removeControlCode($this->base['ngwd']);
		if (!mb_check_encoding($this->base['shwd'], 'UTF-8')) $this->base['shwd'] = '';
		if (!mb_check_encoding($this->base['ngwd'], 'UTF-8')) $this->base['ngwd'] = '';
		if (!preg_match('/^(0|1)$/', $this->base['and'])) $this->base['and'] = '1';
		if (!preg_match('/^(a|u|d)$/', $this->base['r'])) $this->base['r'] = 'a';
		if (!preg_match('/^\d+$/', $this->base['r_dv'])) $this->base['r_dv'] = '';
		if (($this->base['r'] == 'd') && !$this->base['r_dv']) $this->base['r'] = 'a';
		if (!preg_match('/^(0|1)$/', $this->base['sw_b'])) $this->base['sw_b'] = '1';
		if (!preg_match('/^(0|1)$/', $this->base['sw_o'])) $this->base['sw_o'] = '1';
		if (!preg_match('/^(0|1)$/', $this->base['sw_d'])) $this->base['sw_d'] = '1';
		if (!preg_match('/^(0|1)$/', $this->base['sw_h'])) $this->base['sw_h'] = '1';
		if (!preg_match('/^(u|e|n|r)$/', $this->base['o'])) $this->base['o'] = 'u';
		if (!preg_match('/^(0|1)$/', $this->base['rv'])) $this->base['rv'] = '0';
		if (!preg_match('/^(10|20|30|40|50)$/', $this->base['c'])) $this->base['c'] = '10';
		if (!preg_match('/^\d+$/', $this->base['p'])) $this->base['p'] = '0';
		if (!preg_match('/^[a-z]{2,4}$/', $this->base['a'])) $this->base['a'] = 'sh';
	}
	/***********************************************************
	 * 検索オプション項目に格納された値が「数字」または「数字,数字」
	 * という形式かチェックし、NGなら条件から削除する。
	 * ---------------------------------------------------------
	 * @param  なし
	 * @return なし
	 ***********************************************************/
	private function checkOptCondition() {
		foreach($this->option as $group=>$array) { //group=ok,ng
			foreach($array as $key=>$items) { //key=条件の名称
				foreach($items as $index=>$val) {
					if (!preg_match('/^(\d+|\d+,\d+)$/', $this->option[$group][$key][$index])) {
						unset($this->option[$group][$key][$index]);
					}
				}
			}
		}
	}
	/***********************************************************
	 * 検索結果の上部メニュー用に、表示条件を組み立てる。
	 * ---------------------------------------------------------
	 * @param  なし
	 * @return なし
	 ***********************************************************/
	public function buildMenuParam() {
		$arr = array();
		foreach(array('sw_b','sw_o','sw_d','sw_h','o','rv','c') as $key){
			if ($this->base[$key] != $this->base_default[$key]) $arr[] = $key.'='.$this->base[$key];
		}
		return implode('&amp;', $arr);
	}


	public function buildBaseParam($out_params = array()) {
		return implode('&amp;', $this->buildBaseParamSub($out_params));
	}
	private function buildBaseParamSub($out_params = array()) {
		$param = array();
		foreach($this->base as $key=>$val) {
			if (in_array($key, $out_params)) continue;
			if ($val == $this->base_default[$key]) continue;
			$param[] = $key.'='.$val;
		}
		return $param;
	}
	public function buildReqParam($out_params = array()) {
		$param = $this->buildBaseParamSub($out_params);
		foreach($this->option['ok'] as $key=>$items) {
			if (in_array($key, $out_params)) continue;
			$param[] = $key.'='.implode('x', $items);
		}
		foreach($this->option['ng'] as $key=>$items) {
			if (in_array($key, $out_params)) continue;
			$param[] = $key.'_n='.implode('x', $items);
		}
		return implode('&amp;', $param);
	}
}
